
//基础数据转换
function parseBase(_content: string | Buffer) {
	//最终返回结果
	const result = {} as any
	//定义块
	let block: any
	//此函数用于加入值到结果
	(_content + '')
		//删除注释
		.replace(/[;#][\s\S]*?\r?\n/g, '\n')
		//按行分割
		.split(/\r?\n/)
		//trim + filter
		.map(str => str.trim()).filter(str => !!str)
		//单个处理
		.forEach(str => {
			let match: Array<string> | null
			//块
			if (match = str.match(/^\[([\s\S]+)?\]$/)) {
				//得到块名称，并按照点分割，并逐个处理
				block = result
				match[1].trim().split('.').forEach(item => {
					//路径处理
					block = block[item] = block[item] || {}
				})
			}
			else if (match = str.match(/^([\s\S]+)=([\s\S]+)?$/)) {
				let [_, key, value] = match
				key = key.trim()
				value = value && value.trim()
				//块校验
				if (!block) throw new Error(`Can't find block for item [${key}]`)
				//存储值
				//1)数组格式1
				if (/\[\]$/.test(key)) {
					key = key.substr(0, key.length - 2)
					block[key] = block[key] || []
					block[key].push(value)
				}
				//2)数组格式2
				else if (value && value.indexOf(',') >= 0) {
					block[key] = value.split(/,/).map(vi => vi.trim()).filter(vi => !!vi)
				}
				//3)其他
				else {
					block[key] = value
				}
			}
		})
	//现在已经转换完毕了
	return result
}

//细节处理
function parseDetail(data: any, parent: any, key: any): string {
	if (typeof data == 'string') {
		if (data == 'true' || data == 'false') {
			if (parent && (key !== undefined || key !== null)) parent[key] = data == 'true'
			return 'boolean'
		}
		if (/^\d+(\.\d+)?$/.test(data)) {
			if (parent && (key !== undefined || key !== null)) parent[key] = (data as any) * 1
			return 'number'
		}
		else {
			if (parent && (key !== undefined || key !== null)) parent[key] = data.replace(/^"/, '').replace(/"$/, '')
			return 'string'
		}
	}
	else if (data instanceof Array) {
		if (data.length <= 0) return 'Array<any>'
		return `[${data.map((d, N) => parseDetail(d, data, N)).join(', ')}]`
	}
	else {
		return `{\n\t${Object.keys(data).map(key => {
			return `${key}: ${parseDetail(data[key], data, key).replace(/\n/g, '\n\t')}`
		}).join('\n\t')}\n`
	}
}

/**
 * 解码ini文件
 * @param content 数据内容
 */
export function decode(content: string | Buffer) {
	const base = parseBase(content)
	const dts = parseDetail(base, null, null)
	return {
		json: base,
		dts: dts,
	}
}